# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import sys

from PyQt4.QtCore import Qt
from PyQt4.QtGui import QAction, QMenu
from PyQt4.QtTest import QTest

from ....automate import AutomationError, IAutomatedActionTrigger
from ....model import adapt, Adapter, observe
from ....ui import IAction, IActionCollection, IMenu

from ..widget_adapter import WidgetAdapter

from .place_holder import PlaceHolder


@adapt(QMenu, to=IMenu)
class QMenuIMenuAdapter(WidgetAdapter):
    """ The QMenuIMenuAdapter adapts a :class:`~PyQt4.QtGui.QMenu` to the
    :class:`~dip.pui.IMenu` interface.
    """

    @IMenu.title.getter
    def title(self):
        """ Invoked to get the menu's title. """

        return self.adaptee.title()

    @title.setter
    def title(self, value):
        """ Invoked to set the menu's title. """

        self.adaptee.setTitle(value)

    @IMenu.visible.getter
    def visible(self):
        """ Invoked to get the menu's visibility. """

        return self.adaptee.menuAction().isVisible()

    @visible.setter
    def visible(self, value):
        """ Invoked to set the menu's visibility. """

        self.adaptee.menuAction().setVisible(value)

    @IMenu.views.getter
    def views(self):
        """ Invoked to get the views. """

        views = []
        for action in self.adaptee.actions():
            if action.isSeparator():
                item = ''
            elif isinstance(action, PlaceHolder):
                if action.occupied:
                    continue

                item = unicode(action.objectName())
            else:
                submenu = action.menu()
                item = action if submenu is None else submenu

            views.append(item)

        return views

    @views.setter
    def views(self, value):
        """ Invoked to set the views. """

        # We handle this by observing the change.

    @observe('views')
    def _views_changed(self, change):

        menu = self.adaptee

        # FIXME: Remove old views.

        for item in change.new:
            if isinstance(item, QAction):
                menu.addAction(item)
            elif isinstance(item, QMenu):
                menu.addMenu(item)
            elif item != '':
                menu.addAction(PlaceHolder(item, menu))
            else:
                menu.addSeparator()

        # Set the initial visibility.
        self._update_visibility()

    def add_action(self, action):
        """ Try and add an action or action collection to the menu.

        :param action:
            is the action.
        :return:
            ``True`` if the action was added.  ``False`` is returned if the
            menu couldn't accommodate the action's desired positioning.
        """

        if isinstance(action, IActionCollection):
            iaction_collection = IActionCollection(action)

            # Within takes precedence.
            if iaction_collection.within == self.id:
                # Create a sub-menu if needed.
                if iaction_collection.text != '':
                    menu = QMenu(iaction_collection.text, self.adaptee,
                            objectName=iaction_collection.id)
                    self.adaptee.addMenu(menu)
                else:
                    menu = self.adaptee

                for action in iaction_collection.actions:
                    menu.addAction(PlaceHolder.as_QAction(action, self.adaptee))
            else:
                if not PlaceHolder.add_action_collection(action, self.adaptee):
                    return False
        else:
            # Within takes precedence.
            if IAction(action).within == self.id:
                self.adaptee.addAction(action)
            else:
                if not PlaceHolder.add_action(action, self.adaptee):
                    return False

        self._update_visibility()

        return True

    def _update_visibility(self):
        """ Update the visibility of the menu. """

        visible = False

        for action in self.adaptee.actions():
            if action.isSeparator():
                continue

            submenu = action.menu()
            if submenu is None:
                # On OS/X the quit action is in the system menu so ignore it.
                if sys.platform == 'darwin' and action.objectName() == 'dip.ui.actions.quit':
                    continue

                if not isinstance(action, PlaceHolder):
                    visible = True
            else:
                if IMenu(submenu).visible:
                    visible = True

        self.visible = visible


@adapt(QMenu, to=IAutomatedActionTrigger)
class QMenuIAutomatedActionTriggerAdapter(Adapter):
    """ The QMenuIAutomatedActionTriggerAdapter adapts a
    :class:`~PyQt4.QtGui.QMenu` to the
    :class:`~dip.shell.IAutomatedActionTrigger` interface.
    """

    def simulate_trigger(self, id, delay, action_id):
        """ Simulate the user triggering a menu action.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param action_id:
            is the identifier of the action.
        """

        menu = self.adaptee

        # Find the visible menu item that contains the action.
        for action in menu.actions():
            if action.isVisible() and action.objectName() == action_id:
                break
        else:
            raise AutomationError(id, 'trigger',
                    "unknown menu id '{0}'".format(action_id))

        pos = menu.actionGeometry(action).center()

        QTest.mouseClick(menu, Qt.LeftButton, pos=pos, delay=delay)
